@@ -1,7 +1,61 @@
-from typing import Any, Optional
+from typing import Any, Optional, Union, Dict
 
+from chia.types.blockchain_format.sized_bytes import bytes32
 import click
 
+from chia.util.network import is_trusted_inner
+
+
+async def print_connections(client, time, NodeType, trusted_peers: Dict):
+    connections = await client.get_connections()
+    print("Connections:")
+    print("Type      IP                                     Ports       NodeID      Last Connect" + "      MiB Up|Dwn")
+    for con in connections:
+        last_connect_tuple = time.struct_time(time.localtime(con["last_message_time"]))
+        last_connect = time.strftime("%b %d %T", last_connect_tuple)
+        mb_down = con["bytes_read"] / (1024 * 1024)
+        mb_up = con["bytes_written"] / (1024 * 1024)
+
+        host = con["peer_host"]
+        # Strip IPv6 brackets
+        host = host.strip("[]")
+
+        trusted: bool = is_trusted_inner(host, con["node_id"], trusted_peers, False)
+        # Nodetype length is 9 because INTRODUCER will be deprecated
+        if NodeType(con["type"]) is NodeType.FULL_NODE:
+            peak_height = con.get("peak_height", None)
+            connection_peak_hash = con.get("peak_hash", None)
+            if connection_peak_hash is None:
+                connection_peak_hash = "No Info"
+            else:
+                if connection_peak_hash.startswith(("0x", "0X")):
+                    connection_peak_hash = connection_peak_hash[2:]
+                connection_peak_hash = f"{connection_peak_hash[:8]}..."
+            con_str = (
+                f"{NodeType(con['type']).name:9} {host:38} "
+                f"{con['peer_port']:5}/{con['peer_server_port']:<5}"
+                f" {con['node_id'].hex()[:8]}... "
+                f"{last_connect}  "
+                f"{mb_up:7.1f}|{mb_down:<7.1f}"
+                f"\n                                                 "
+            )
+            if peak_height is not None:
+                con_str += f"-Height: {peak_height:8.0f}    -Hash: {connection_peak_hash}"
+            else:
+                con_str += f"-Height: No Info    -Hash: {connection_peak_hash}"
+            # Only show when Trusted is True
+            if trusted:
+                con_str += f"    -Trusted: {trusted}"
+        else:
+            con_str = (
+                f"{NodeType(con['type']).name:9} {host:38} "
+                f"{con['peer_port']:5}/{con['peer_server_port']:<5}"
+                f" {con['node_id'].hex()[:8]}... "
+                f"{last_connect}  "
+                f"{mb_up:7.1f}|{mb_down:<7.1f}"
+            )
+        print(con_str)
+
 
 async def show_async(
     rpc_port: Optional[int],
@@ -14,10 +68,8 @@ async def show_async(
     block_by_header_hash: str,
 ) -> None:
     import aiohttp
-    import time
     import traceback
-
-    from time import localtime, struct_time
+    import time
     from typing import List, Optional
     from chia.consensus.block_record import BlockRecord
     from chia.rpc.full_node_rpc_client import FullNodeRpcClient
@@ -43,12 +95,22 @@ async def show_async(
                 print("There is no blockchain found yet. Try again shortly")
                 return None
             peak: Optional[BlockRecord] = blockchain_state["peak"]
+            node_id = blockchain_state["node_id"]
             difficulty = blockchain_state["difficulty"]
             sub_slot_iters = blockchain_state["sub_slot_iters"]
             synced = blockchain_state["sync"]["synced"]
             sync_mode = blockchain_state["sync"]["sync_mode"]
             total_iters = peak.total_iters if peak is not None else 0
             num_blocks: int = 10
+            network_name = config["selected_network"]
+            genesis_challenge = config["farmer"]["network_overrides"]["constants"][network_name]["GENESIS_CHALLENGE"]
+            full_node_port = config["full_node"]["port"]
+            full_node_rpc_port = config["full_node"]["rpc_port"]
+
+            print(f"Network: {network_name}    Port: {full_node_port}   Rpc Port: {full_node_rpc_port}")
+            print(f"Node ID: {node_id}")
+
+            print(f"Genesis Challenge: {genesis_challenge}")
 
             if synced:
                 print("Current Blockchain Status: Full Node Synced")
@@ -56,7 +118,10 @@ async def show_async(
             elif peak is not None and sync_mode:
                 sync_max_block = blockchain_state["sync"]["sync_tip_height"]
                 sync_current_block = blockchain_state["sync"]["sync_progress_height"]
-                print(f"Current Blockchain Status: Syncing {sync_current_block}/{sync_max_block}.")
+                print(
+                    f"Current Blockchain Status: Syncing {sync_current_block}/{sync_max_block} "
+                    f"({sync_max_block - sync_current_block} behind)."
+                )
                 print("Peak: Hash:", peak.header_hash if peak is not None else "")
             elif peak is not None:
                 print(f"Current Blockchain Status: Not Synced. Peak height: {peak.height}")
@@ -73,7 +138,7 @@ async def show_async(
                     while curr is not None and not curr.is_transaction_block:
                         curr = await client.get_block_record(curr.prev_hash)
                     peak_time = curr.timestamp
-                peak_time_struct = struct_time(localtime(peak_time))
+                peak_time_struct = time.struct_time(time.localtime(peak_time))
 
                 print(
                     "      Time:",
@@ -104,48 +169,8 @@ async def show_async(
             if show_connections:
                 print("")
         if show_connections:
-            connections = await client.get_connections()
-            print("Connections:")
-            print(
-                "Type      IP                                     Ports       NodeID      Last Connect"
-                + "      MiB Up|Dwn"
-            )
-            for con in connections:
-                last_connect_tuple = struct_time(localtime(con["last_message_time"]))
-                last_connect = time.strftime("%b %d %T", last_connect_tuple)
-                mb_down = con["bytes_read"] / (1024 * 1024)
-                mb_up = con["bytes_written"] / (1024 * 1024)
-
-                host = con["peer_host"]
-                # Strip IPv6 brackets
-                if host[0] == "[":
-                    host = host[1:39]
-                # Nodetype length is 9 because INTRODUCER will be deprecated
-                if NodeType(con["type"]) is NodeType.FULL_NODE:
-                    peak_height = con["peak_height"]
-                    peak_hash = con["peak_hash"]
-                    if peak_hash is None:
-                        peak_hash = "No Info"
-                    if peak_height is None:
-                        peak_height = 0
-                    con_str = (
-                        f"{NodeType(con['type']).name:9} {host:38} "
-                        f"{con['peer_port']:5}/{con['peer_server_port']:<5}"
-                        f" {con['node_id'].hex()[:8]}... "
-                        f"{last_connect}  "
-                        f"{mb_up:7.1f}|{mb_down:<7.1f}"
-                        f"\n                                                 "
-                        f"-SB Height: {peak_height:8.0f}    -Hash: {peak_hash[2:10]}..."
-                    )
-                else:
-                    con_str = (
-                        f"{NodeType(con['type']).name:9} {host:38} "
-                        f"{con['peer_port']:5}/{con['peer_server_port']:<5}"
-                        f" {con['node_id'].hex()[:8]}... "
-                        f"{last_connect}  "
-                        f"{mb_up:7.1f}|{mb_down:<7.1f}"
-                    )
-                print(con_str)
+            trusted_peers: Dict = config["full_node"].get("trusted_peers", {})
+            await print_connections(client, time, NodeType, trusted_peers)
             # if called together with state, leave a blank line
             if state:
                 print("")
@@ -203,8 +228,8 @@ async def show_async(
                     difficulty = block.weight
                 if block.is_transaction_block:
                     assert full_block.transactions_info is not None
-                    block_time = struct_time(
-                        localtime(
+                    block_time = time.struct_time(
+                        time.localtime(
                             full_block.foliage_transaction_block.timestamp
                             if full_block.foliage_transaction_block
                             else None
@@ -212,7 +237,7 @@ async def show_async(
                     )
                     block_time_string = time.strftime("%a %b %d %Y %T %Z", block_time)
                     cost = str(full_block.transactions_info.cost)
-                    tx_filter_hash = "Not a transaction block"
+                    tx_filter_hash: Union[str, bytes32] = "Not a transaction block"
                     if full_block.foliage_transaction_block:
                         tx_filter_hash = full_block.foliage_transaction_block.filter_hash
                     fees: Any = block.fees
