@@ -16,6 +16,7 @@ from chia.plotting.util import (
     find_duplicate_plot_IDs,
     parse_plot_info,
 )
+from chia.util.bech32m import encode_puzzle_hash
 from chia.util.config import load_config
 from chia.util.hash import std_hash
 from chia.util.keychain import Keychain
@@ -25,16 +26,16 @@ log = logging.getLogger(__name__)
 
 
 def plot_refresh_callback(event: PlotRefreshEvents, refresh_result: PlotRefreshResult):
-    log.info(f"event: {event.name}, loaded {refresh_result.loaded} plots, {refresh_result.remaining} remaining")
+    log.info(f"event: {event.name}, loaded {len(refresh_result.loaded)} plots, {refresh_result.remaining} remaining")
 
 
 def check_plots(root_path, num, challenge_start, grep_string, list_duplicates, debug_show_memo):
     config = load_config(root_path, "config.yaml")
+    address_prefix = config["network_overrides"]["config"][config["selected_network"]]["address_prefix"]
     plot_refresh_parameter: PlotsRefreshParameter = PlotsRefreshParameter(batch_sleep_milliseconds=0)
     plot_manager: PlotManager = PlotManager(
         root_path,
         match_str=grep_string,
-        show_memo=debug_show_memo,
         open_no_key_filenames=True,
         refresh_parameter=plot_refresh_parameter,
         refresh_callback=plot_refresh_callback,
@@ -102,7 +103,11 @@ def check_plots(root_path, num, challenge_start, grep_string, list_duplicates, d
         for plot_path, plot_info in plot_manager.plots.items():
             pr = plot_info.prover
             log.info(f"Testing plot {plot_path} k={pr.get_size()}")
-            log.info(f"\tPool public key: {plot_info.pool_public_key}")
+            if plot_info.pool_public_key is not None:
+                log.info(f"\t{'Pool public key:':<23} {plot_info.pool_public_key}")
+            if plot_info.pool_contract_puzzle_hash is not None:
+                pca: str = encode_puzzle_hash(plot_info.pool_contract_puzzle_hash, address_prefix)
+                log.info(f"\t{'Pool contract address:':<23} {pca}")
 
             # Look up local_sk from plot to save locked memory
             (
@@ -111,8 +116,8 @@ def check_plots(root_path, num, challenge_start, grep_string, list_duplicates, d
                 local_master_sk,
             ) = parse_plot_info(pr.get_memo())
             local_sk = master_sk_to_local_sk(local_master_sk)
-            log.info(f"\tFarmer public key: {farmer_public_key}")
-            log.info(f"\tLocal sk: {local_sk}")
+            log.info(f"\t{'Farmer public key:' :<23} {farmer_public_key}")
+            log.info(f"\t{'Local sk:' :<23} {local_sk}")
             total_proofs = 0
             caught_exception: bool = False
             for i in range(num_start, num_end):
@@ -191,3 +196,10 @@ def check_plots(root_path, num, challenge_start, grep_string, list_duplicates, d
             f"is not on this machine. The farmer private key must be in the keychain in order to "
             f"farm them, use 'chia keys' to transfer keys. The pool public keys must be in the config.yaml"
         )
+
+    if debug_show_memo:
+        plot_memo_str: str = "Plot Memos:\n"
+        with plot_manager:
+            for path, plot in plot_manager.plots.items():
+                plot_memo_str += f"{path}: {plot.prover.get_memo().hex()}\n"
+        log.info(plot_memo_str)
