@@ -3,7 +3,7 @@ from typing import Dict, List, Tuple
 
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.condition_with_args import ConditionWithArgs
-from chia.util.condition_tools import ConditionOpcode
+from chia.types.condition_opcodes import ConditionOpcode
 from chia.util.streamable import Streamable, streamable
 
 
@@ -18,5 +18,6 @@ class NPC(Streamable):
     def condition_dict(self):
         d: Dict[ConditionOpcode, List[ConditionWithArgs]] = {}
         for opcode, l in self.conditions:
+            assert opcode not in d
             d[opcode] = l
         return d
