@@ -2,9 +2,7 @@ from typing import Any
 
 import aiosqlite
 
-from chia.util.byte_types import hexstr_to_bytes
 from chia.util.db_wrapper import DBWrapper
-from chia.util.streamable import Streamable
 
 
 class KeyValStore:
@@ -21,7 +19,7 @@ class KeyValStore:
         self.db_wrapper = db_wrapper
         self.db_connection = db_wrapper.db
         await self.db_connection.execute(
-            ("CREATE TABLE IF NOT EXISTS key_val_store(" " key text PRIMARY KEY," " value text)")
+            "CREATE TABLE IF NOT EXISTS key_val_store(" " key text PRIMARY KEY," " value blob)"
         )
 
         await self.db_connection.execute("CREATE INDEX IF NOT EXISTS name on key_val_store(key)")
@@ -34,7 +32,7 @@ class KeyValStore:
         await cursor.close()
         await self.db_connection.commit()
 
-    async def get_object(self, key: str, type: Any) -> Any:
+    async def get_object(self, key: str, object_type: Any) -> Any:
         """
         Return bytes representation of stored object
         """
@@ -46,16 +44,33 @@ class KeyValStore:
         if row is None:
             return None
 
-        return type.from_bytes(hexstr_to_bytes(row[1]))
+        return object_type.from_bytes(row[1])
 
-    async def set_object(self, key: str, obj: Streamable):
+    async def set_object(self, key: str, obj: Any, in_transaction=False):
         """
-        Adds object to key val store
+        Adds object to key val store. Obj MUST support __bytes__ and bytes() methods.
         """
-        async with self.db_wrapper.lock:
+        if not in_transaction:
+            await self.db_wrapper.lock.acquire()
+
+        try:
             cursor = await self.db_connection.execute(
                 "INSERT OR REPLACE INTO key_val_store VALUES(?, ?)",
-                (key, bytes(obj).hex()),
+                (key, bytes(obj)),
             )
             await cursor.close()
-            await self.db_connection.commit()
+        finally:
+            if not in_transaction:
+                await self.db_connection.commit()
+                self.db_wrapper.lock.release()
+
+    async def remove_object(self, key: str, in_transaction=False):
+        if not in_transaction:
+            await self.db_wrapper.lock.acquire()
+        try:
+            cursor = await self.db_connection.execute("DELETE FROM key_val_store where key=?", (key,))
+            await cursor.close()
+        finally:
+            if not in_transaction:
+                await self.db_connection.commit()
+                self.db_wrapper.lock.release()
