@@ -6,7 +6,7 @@ from clvm_tools import binutils
 from chia.consensus.block_rewards import calculate_base_farmer_reward, calculate_pool_reward
 from chia.rpc.full_node_rpc_client import FullNodeRpcClient
 from chia.types.blockchain_format.program import Program
-from chia.types.coin_solution import CoinSolution
+from chia.types.coin_spend import CoinSpend
 from chia.types.condition_opcodes import ConditionOpcode
 from chia.types.spend_bundle import SpendBundle
 from chia.util.bech32m import decode_puzzle_hash
@@ -18,10 +18,8 @@ from chia.util.ints import uint32, uint16
 
 def print_conditions(spend_bundle: SpendBundle):
     print("\nConditions:")
-    for coin_solution in spend_bundle.coin_solutions:
-        result = Program.from_bytes(bytes(coin_solution.puzzle_reveal)).run(
-            Program.from_bytes(bytes(coin_solution.solution))
-        )
+    for coin_spend in spend_bundle.coin_spends:
+        result = Program.from_bytes(bytes(coin_spend.puzzle_reveal)).run(Program.from_bytes(bytes(coin_spend.solution)))
         error, result_human = parse_sexp_to_conditions(result)
         assert error is None
         assert result_human is not None
@@ -65,8 +63,8 @@ async def main() -> None:
 
         p_solution = Program.to(binutils.assemble("()"))
 
-        sb_farmer = SpendBundle([CoinSolution(farmer_prefarm, p_farmer_2, p_solution)], G2Element())
-        sb_pool = SpendBundle([CoinSolution(pool_prefarm, p_pool_2, p_solution)], G2Element())
+        sb_farmer = SpendBundle([CoinSpend(farmer_prefarm, p_farmer_2, p_solution)], G2Element())
+        sb_pool = SpendBundle([CoinSpend(pool_prefarm, p_pool_2, p_solution)], G2Element())
 
         print("\n\n\nConditions")
         print_conditions(sb_pool)
