@@ -3,17 +3,21 @@
 # Run from the current directory.
 
 import argparse
+import sys
+
 import testconfig
 import logging
-import subprocess
 from pathlib import Path
-from typing import List
+from typing import Dict, List
+
+root_path = Path(__file__).parent.resolve()
 
 
-def subdirs(root_dirs: List[str]) -> List[Path]:
+def subdirs() -> List[Path]:
     dirs: List[Path] = []
-    for r in root_dirs:
-        dirs.extend(Path(r).rglob("**/"))
+    for r in root_path.iterdir():
+        if r.is_dir():
+            dirs.extend(Path(r).rglob("**/"))
     return [d for d in dirs if not (any(c.startswith("_") for c in d.parts) or any(c.startswith(".") for c in d.parts))]
 
 
@@ -24,22 +28,20 @@ def module_dict(module):
 def dir_config(dir):
     import importlib
 
-    module_name = str(dir).replace("/", ".") + ".config"
+    module_name = ".".join([*dir.relative_to(root_path).parts, "config"])
     try:
         return module_dict(importlib.import_module(module_name))
     except ModuleNotFoundError:
         return {}
 
 
-def read_file(filename):
-    with open(filename) as f:
-        return f.read()
-    return None
+def read_file(filename: Path) -> str:
+    return filename.read_bytes().decode("utf8")
 
 
 # Input file
 def workflow_yaml_template_text(os):
-    return Path(f"runner-templates/build-test-{os}").read_text()
+    return read_file(Path(root_path / f"runner_templates/build-test-{os}"))
 
 
 # Output files
@@ -49,7 +51,7 @@ def workflow_yaml_file(dir, os, test_name):
 
 # String function from test dir to test name
 def test_name(dir):
-    return str(dir).replace("/", "-")
+    return "-".join(dir.relative_to(root_path).parts)
 
 
 def transform_template(template_text, replacements):
@@ -59,28 +61,17 @@ def transform_template(template_text, replacements):
     return t
 
 
-def test_files_in_dir(dir):
-    g = dir.glob("test_*.py")
-    return [] if g is None else [f for f in g]
-
-
-# -----
-
-default_replacements = {
-    "INSTALL_TIMELORD": read_file("runner-templates/install-timelord.include.yml").rstrip(),
-    "CHECKOUT_TEST_BLOCKS_AND_PLOTS": read_file("runner-templates/checkout-test-plots.include.yml").rstrip(),
-    "TEST_DIR": "",
-    "TEST_NAME": "",
-    "PYTEST_PARALLEL_ARGS": "",
-}
-
-# -----
-
-
 # Replace with update_config
-def generate_replacements(defaults, conf, dir, test_files):
-    assert len(test_files) > 0
-    replacements = dict(defaults)
+def generate_replacements(conf, dir):
+    replacements = {
+        "INSTALL_TIMELORD": read_file(Path(root_path / "runner_templates/install-timelord.include.yml")).rstrip(),
+        "CHECKOUT_TEST_BLOCKS_AND_PLOTS": read_file(
+            Path(root_path / "runner_templates/checkout-test-plots.include.yml")
+        ).rstrip(),
+        "TEST_DIR": "",
+        "TEST_NAME": "",
+        "PYTEST_PARALLEL_ARGS": "",
+    }
 
     if not conf["checkout_blocks_and_plots"]:
         replacements[
@@ -92,11 +83,8 @@ def generate_replacements(defaults, conf, dir, test_files):
         replacements["PYTEST_PARALLEL_ARGS"] = " -n auto"
     if conf["job_timeout"]:
         replacements["JOB_TIMEOUT"] = str(conf["job_timeout"])
-    test_paths = ["tests/" + str(f) for f in test_files]
-    # We have to list the test files individually until pytest has the
-    # option to only collect tests in the named dir, and not those below
-    replacements["TEST_DIR"] = " ".join(sorted(test_paths))
-    replacements["TEST_NAME"] = test_name(str(dir))
+    replacements["TEST_DIR"] = "/".join([*dir.relative_to(root_path.parent).parts, "test_*.py"])
+    replacements["TEST_NAME"] = test_name(dir)
     if "test_name" in conf:
         replacements["TEST_NAME"] = conf["test_name"]
     for var in conf["custom_vars"]:
@@ -116,7 +104,7 @@ def update_config(parent, child):
 
 
 def dir_path(string):
-    p = Path(string)
+    p = Path(root_path / string)
     if p.is_dir():
         return p
     else:
@@ -126,6 +114,7 @@ def dir_path(string):
 # args
 arg_parser = argparse.ArgumentParser(description="Build github workflows")
 arg_parser.add_argument("--output-dir", "-d", default="../.github/workflows", type=dir_path)
+arg_parser.add_argument("--fail-on-update", "-f", action="store_true")
 arg_parser.add_argument("--verbose", "-v", action="store_true")
 args = arg_parser.parse_args()
 
@@ -133,21 +122,28 @@ if args.verbose:
     logging.basicConfig(format="%(asctime)s:%(message)s", level=logging.DEBUG)
 
 # main
-test_dirs = subdirs(testconfig.root_test_dirs)
+test_dirs = subdirs()
+current_workflows: Dict[Path, str] = {file: read_file(file) for file in args.output_dir.iterdir()}
+changed: bool = False
 
 for os in testconfig.oses:
     template_text = workflow_yaml_template_text(os)
     for dir in test_dirs:
-        test_files = test_files_in_dir(dir)
-        if len(test_files) == 0:
+        if len([f for f in Path(root_path / dir).glob("test_*.py")]) == 0:
             logging.info(f"Skipping {dir}: no tests collected")
             continue
         conf = update_config(module_dict(testconfig), dir_config(dir))
-        replacements = generate_replacements(default_replacements, conf, dir, test_files)
+        replacements = generate_replacements(conf, dir)
         txt = transform_template(template_text, replacements)
         logging.info(f"Writing {os}-{test_name(dir)}")
-        workflow_yaml_file(args.output_dir, os, test_name(dir)).write_text(txt)
+        workflow_yaml_path: Path = workflow_yaml_file(args.output_dir, os, test_name(dir))
+        if workflow_yaml_path not in current_workflows or current_workflows[workflow_yaml_path] != txt:
+            changed = True
+        workflow_yaml_path.write_bytes(txt.encode("utf8"))
 
-out = subprocess.run(["git", "diff", args.output_dir])
-if out.stdout:
-    print(out.stdout)
+if changed:
+    print("New workflow updates available.")
+    if args.fail_on_update:
+        sys.exit(1)
+else:
+    print("Nothing to do.")
